/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package cmd

import (
	"fmt"

	"github.com/manifoldco/promptui"
	"github.com/spf13/cobra"
	"go.uploadedlobster.com/scotty/internal/backends"
	"go.uploadedlobster.com/scotty/internal/cli"
	"go.uploadedlobster.com/scotty/internal/config"
)

// addCmd represents the add command
var addCmd = &cobra.Command{
	Use:   "add",
	Short: "Add a service configuration",
	Long:  `Add a service configuration.`,
	Run: func(cmd *cobra.Command, args []string) {
		// Select backend
		sel := promptui.Select{
			Label: "Backend",
			Items: backends.GetBackends(),
			Size:  10,
		}
		_, backend, err := sel.Run()
		cobra.CheckErr(err)

		// Set service name
		prompt := promptui.Prompt{
			Label:    "Service name",
			Validate: config.ValidateKey,
			Default:  backend,
		}
		name, err := prompt.Run()
		cobra.CheckErr(err)

		// Prepate service config
		service := config.ServiceConfig{
			Name:    name,
			Backend: backend,
		}

		// Additional options
		err = extraOptions(&service)
		cobra.CheckErr(err)

		// Save the service config
		err = service.Save()
		cobra.CheckErr(err)
		fmt.Printf("Saved service %v using backend %v\n", service.Name, service.Backend)
	},
}

func init() {
	configCmd.AddCommand(addCmd)

	// Here you will define your flags and configuration settings.

	// Cobra supports Persistent Flags which will work for this command
	// and all subcommands, e.g.:
	// addCmd.PersistentFlags().String("foo", "", "A help for foo")

	// Cobra supports local flags which will only run when this command
	// is called directly, e.g.:
	// addCmd.Flags().BoolP("toggle", "t", false, "Help message for toggle")
}

func extraOptions(config *config.ServiceConfig) error {
	backend, err := backends.BackendByName(config.Backend)
	if err != nil {
		return err
	}
	opts := backend.Options()
	if opts == nil {
		return nil
	}

	values := make(map[string]any, len(*opts))
	for _, opt := range *opts {
		val, err := cli.Prompt(opt)
		if err != nil {
			return err
		}
		values[opt.Name] = val

	}

	config.ConfigValues = values
	return nil
}
